#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import snack
from snack import SnackScreen, Checkbox, Grid, Form, Label, CompactButton
from messagebox import MessageBox

class ChoiceForm(object):
    """
    Форма для выбора действий при сборе системной информации
    """
    
    def callBackAll(self, checkBoxList):
        """
        Срабатывает при выборе первого флажка. Выбираются все остальные флажки
        checkBoxList - Список флажков
        """
        mainCheckBox = checkBoxList.pop(0)
        for checkBox in checkBoxList:
            checkBox.setValue("*" if mainCheckBox.selected() else " ")
        checkBoxList.insert(0, mainCheckBox)
        
    def callBackOne(self, checkBoxList):
        """
        Срабатывает при выборе всех флажков, кроме первого. Проверяется первый флажок
        checkBoxList - Список флажков
        """
        mainCheckBox = checkBoxList.pop(0)
        rez = True
        for checkBox in checkBoxList:
            rez = rez and checkBox.selected()
        mainCheckBox.setValue("*" if rez else " ")
        checkBoxList.insert(0, mainCheckBox)
    
    def makeResult(self, checkBoxList):
        """
        Создаёт список операций для выполнения
        checkBoxList - Список флажков
        """
        operations = []
        for i in range(1, len(checkBoxList)):
            operations.append(checkBoxList[i].selected())
        return operations
    
    def __init__(self, screen):
        """
        screen - экран, на котором будет появляться данное окно
        """
        self.screen = screen
    
    def show(self):
        """
        Конструирует и показывает окно
        """
        
        labelText =  Label("Выберите данные, которые необходимо собрать.")
        labelEmpty = Label("                                            ")
        
        gridLabel = Grid(2, 8)
        gridLabel.setField(labelText, 0, 0)
        gridLabel.setField(labelEmpty, 0, 1)
       
        checkboxAll =  Checkbox("Выбрать всё", isOn=0)
        checkboxDict = Checkbox('БД "Справочники"', isOn=1)
        checkboxDoc =  Checkbox('БД "Продажи"', isOn=0)
        checkboxConf = Checkbox("Логи Артикс и конф. файлы", isOn=1)
        checkboxSys =  Checkbox("Системная информация", isOn=1)
        
        checkboxlist = [checkboxAll, checkboxDict, checkboxDoc, checkboxConf, checkboxSys]
        
        checkboxAll.setCallback(self.callBackAll, checkboxlist)
        checkboxDict.setCallback(self.callBackOne, checkboxlist)
        checkboxDoc.setCallback(self.callBackOne, checkboxlist)
        checkboxConf.setCallback(self.callBackOne, checkboxlist)
        checkboxSys.setCallback(self.callBackOne, checkboxlist)
        
        gridCheckboxes = Grid(2, 5)
        gridCheckboxes.setField(checkboxAll,  0, 0, anchorLeft=1)
        gridCheckboxes.setField(checkboxDict, 1, 1, (4, 0, 0, 0), anchorLeft=1)
        gridCheckboxes.setField(checkboxDoc,  1, 2, (4, 0, 0, 0), anchorLeft=1)
        gridCheckboxes.setField(checkboxConf, 1, 3, (4, 0, 0, 0), anchorLeft=1)
        gridCheckboxes.setField(checkboxSys,  1, 4, (4, 0, 0, 0), anchorLeft=1)
        
        btnOk = CompactButton(" OK ")
        btnCancel = CompactButton(" Отмена ")
        
        gridButton = Grid(12, 1)
        gridButton.setField(btnOk, 0, 0)
        gridButton.setField(btnCancel, 8, 0)
        
        gridForm = Grid(5, 10)
        gridForm.setField(gridLabel, 0, 0)
        gridForm.setField(gridCheckboxes, 0, 2, anchorLeft = 1)
        gridForm.setField(labelEmpty, 0, 7)
        gridForm.setField(gridButton, 0, 8)

        isContinue = False
        operations = []
        f = Form()
        f.add(gridForm)
        while not isContinue:
            self.screen.gridWrappedWindow(gridForm, "Выбор данных")
            result = f.run()
            if result == btnOk:
            #Проверяем операции
                operations = self.makeResult(checkboxlist)
                if operations.count(True) == 0:
                    MessageBox(self.screen, "Информация", "              Ничего не выбрано").show()  
                else:
                    isContinue = True      
            else:
                operations = []
                isContinue = True
        return operations
        
        
   
   





   



