#!/usr/bin/env python
# -*- coding: utf-8 -*

import os
import subprocess
import sys
import shutil
import tarfile

sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.dialog import dialog
from py_tools.logger import logger
from snack import SnackScreen
from attentionform import AttentionForm
from choiceform import ChoiceForm
from messagebox import MessageBox
from scaleform import ScaleForm

log = logger.Logger()

#Имя рабочей директрии
workingDir = "/linuxcash/logs/backup"

#Имя архива
archiveName = workingDir + "/backup_system_info.tar.gz"

#Имя промежуточной папки для файлов с выводом команд
commandFolder = "./information"

#Имя промежуточной папки для дампов MySQL
sqlFolder = "./sqldumps"

#Имя промежуточной папки для информации о процессоре и оперативной памяти
procFolder = "./proc"

def addFileOrFolderInArchive(tar, source):
    """
    Добавляет файл или папку в архив, если файл или папка существует
    tar - описатель архива
    source - файл или папка, которую необходимо записать в архив
    """
    if os.path.exists(source):
        if os.path.isfile(source):
            tar.add(source, arcname="/backup_system_info" + source.lstrip('.'))
        else:
            tar.add(source + "/", recursive=True, arcname="/backup_system_info" + source.lstrip('.'))
    else:
        log.warn("Файл " + source + " Не существует.")

def getAllPackets(tar):
    """
    Записывает информацию об установленных пакетах в архив
    tar - описатель архива
    """
    log.info("Получение списка всех пакетов, установленных в системе")
    if not os.path.exists(commandFolder):
        os.mkdir(commandFolder)
    subprocess.call(["dpkg -l > " + commandFolder + "/packages"], shell=True)
    addFileOrFolderInArchive(tar, commandFolder + "/packages")
    shutil.rmtree(commandFolder)

def getProcessorInfo(tar):
    """
    Записывает информацию о процессоре в архив
    tar - описатель архива
    """
    log.info("Получение информации о процессоре")
    if not os.path.exists(procFolder):
        os.mkdir(procFolder)
    subprocess.call(["cat /proc/cpuinfo > " + procFolder + "/cpuinfo"], shell=True)
    addFileOrFolderInArchive(tar, procFolder + "/cpuinfo")
    shutil.rmtree(procFolder)


def getRamInfo(tar):
    """
    Записывает информацию об оперативной памяти в архив
    tar - описатель архива
    """
    log.info("Получение информации об оперативной памяти")
    if not os.path.exists(procFolder):
        os.mkdir(procFolder)
    subprocess.call(["cat /proc/meminfo > " + procFolder + "/meminfo"], shell=True)
    addFileOrFolderInArchive(tar, procFolder + "/meminfo")
    shutil.rmtree(procFolder)

def getChipsetInfo(tar):
    """
    Записывает информацию о чипсете в архив
    tar - описатель архива
    """
    log.info("Получение информации о чипсете")
    if not os.path.exists(commandFolder):
        os.mkdir(commandFolder)
    subprocess.call(["lspci -v > " + commandFolder + "/pci"], shell=True)
    subprocess.call(["lsusb -v > " + commandFolder + "/usb"], shell=True)
    subprocess.call(["lshw > " + commandFolder + "/hardware 2>&1"], shell=True)
    addFileOrFolderInArchive(tar, commandFolder)
    shutil.rmtree(commandFolder)

def getMySQLDictionariesDumps(tar):
    """
    Записывает дамп БД "Справочники" в архив
    tar - описатель архива
    """
    if not os.path.exists(sqlFolder):
        os.mkdir(sqlFolder)
    log.info("Получение дампа справочников")
    subprocess.call(["mysqldump -u root dictionaries | gzip > " + sqlFolder + "/dictionaries.sql.gz"], shell=True)
    addFileOrFolderInArchive(tar, sqlFolder)
    shutil.rmtree(sqlFolder)

def getMySQLDocumentsDumps(tar):
    """
    Записывает дамп БД "Продажи" в архив
    tar - описатель архива
    """
    if not os.path.exists(sqlFolder):
        os.mkdir(sqlFolder)
    log.info("Получение дампа продаж")
    subprocess.call(["mysqldump -u root documents | gzip > " + sqlFolder + "/documents.sql.gz"], shell=True)
    addFileOrFolderInArchive(tar, sqlFolder)
    shutil.rmtree(sqlFolder)


def getArtixLogs(tar):
    """
    Записывает логи Артикс и конфигурационные файлы в архив
    tar - описатель архива
    """
    log.info("Получение системной информации")
    addFileOrFolderInArchive(tar, "/etc/X11/xorg.conf")
    addFileOrFolderInArchive(tar, "/linuxcash/cash/data/cash.reg")
    addFileOrFolderInArchive(tar, "/linuxcash/cash/data/tmp/check.img")
    addFileOrFolderInArchive(tar, "/etc/source.list")
    addFileOrFolderInArchive(tar, "/linuxcash/logs/cashlogs/")
    addFileOrFolderInArchive(tar, "/linuxcash/logs/current/")
    addFileOrFolderInArchive(tar, "/linuxcash/logs/defects/")
    addFileOrFolderInArchive(tar, "/linuxcash/logs/keys/")
    addFileOrFolderInArchive(tar, "/linuxcash/logs/lost+found/")
    addFileOrFolderInArchive(tar, "/linuxcash/cash/conf")

def ClearFiles():
    """
    Очищает временные файлы
    """
    log.info("Очистка файлов")
    if os.path.exists(commandFolder):
        if os.path.isdir(commandFolder):
            shutil.rmtree(commandFolder)
    if os.path.exists(sqlFolder):
        if os.path.isdir(sqlFolder):
            shutil.rmtree(sqlFolder)
    if os.path.exists(procFolder):
        if os.path.isdir(procFolder):	
            shutil.rmtree(procFolder)
    if os.path.exists(archiveName):
        if tarfile.is_tarfile(archiveName):
            os.remove(archiveName)
 
def getSysInfo(screen, operations):
    """
    Записывает в архив информацию о системе целиком
    screen - экран, на котором будут появляться окна
    operations - список, позволяющий или не позволяющий операции
    """
    log.info("Получение всей информации в системе")
    tar = tarfile.open(archiveName, "w:gz")
    scaleForm = ScaleForm(screen, 50, "Получение информации", operations.count(True))
    try:
        if operations[0] == True:
            scaleForm.setProgress('Получение информации о БД "справочники"')
            getMySQLDictionariesDumps(tar)
        if operations[1] == True:
            scaleForm.setProgress('Получение информации о БД "Продажи"')
            getMySQLDocumentsDumps(tar)
        if operations[2] == True:
            scaleForm.setProgress("Получение артикс-логов и конфигурационных файлов")
            getArtixLogs(tar)
        if operations[3] == True:
            scaleForm.setProgress("Получение системной информации")
            getAllPackets(tar)
            getProcessorInfo(tar)
            getRamInfo(tar)
            getChipsetInfo(tar)
            addFileOrFolderInArchive(tar, "/var/log")
    except KeyboardInterrupt:
        tar.close()
        ClearFiles()
        raise KeyboardInterrupt
    scaleForm.hide()
    tar.close()
    log.info("Информация о системе получена")


def toChoice(screen):
    """
    Переходит на окно выбора
    """
    operations = ChoiceForm(screen).show()
    if (len(operations) > 0 ):
        getSysInfo(screen, operations)
        MessageBox(screen, "Сообщение",
                           "               Информация собрана.\n" +
                           archiveName).show()
    else:
        screen.popWindow()    
        screen.finish()
        exit(0)

def main():
    """
    Основная функция
    """
    log.info("Запуск сбора системной информации ")
    screen = SnackScreen()
    screen.setColor('CHECKBOX', "blue", "white")
    try:
        if not os.path.exists(workingDir):
            os.makedirs(workingDir)

        if os.path.exists(archiveName) and tarfile.is_tarfile(archiveName):
            if AttentionForm(screen).show():
                toChoice(screen)
            else:
                screen.popWindow()    
                screen.finish()
                exit(0)
        else:
            toChoice(screen)
    except KeyboardInterrupt:
        log.warn("Неожиданное завершение сбора системной информации")
        MessageBox(screen, "Предупреждение", "               " +
                                             "Информация не собрана.\n").show()
    finally:
        screen.popWindow()
        screen.finish()

if __name__ == "__main__":
    os.chdir(os.path.dirname(os.path.abspath(__file__)))
    main()

